@@ -421,8 +421,11 @@ bool RdpSession::start()
     }
 
     if (!m_preferences->shareMedia().isEmpty()) {
-        char *params[2] = {strdup("drive"), m_preferences->shareMedia().toLocal8Bit().data()};
-        freerdp_client_add_device_channel(settings, 1, params);
+        QByteArray name = "drive";
+        QByteArray value = m_preferences->shareMedia().toLocal8Bit();
+
+        char *params[2] = {name.data(), value.data()};
+        freerdp_client_add_device_channel(settings, 2, params);
     }
 
     settings->KeyboardLayout = m_preferences->rdpKeyboardLayout();
