@@ -13,16 +13,20 @@
 #include <QObject>
 #include <QSize>
 
+#include <freerdp/client/cliprdr.h>
 #include <freerdp/freerdp.h>
 
+class RdpClipboard;
 class RdpSession;
 class RdpView;
 class RdpHostPreferences;
+class QMimeData;
 
 struct RdpContext {
     rdpContext _c;
 
     RdpSession *session = nullptr;
+    RdpClipboard *clipboard = nullptr;
 };
 
 struct Certificate {
@@ -95,12 +99,21 @@ public:
 
     bool sendEvent(QEvent *event, QWidget *source);
 
+    void initializeClipboard(RdpContext *rdpC, CliprdrClientContext *cliprdr);
+    void destroyClipboard();
+    bool sendClipboard(const QMimeData *data);
+
     const QImage *videoBuffer() const;
 
     Q_SIGNAL void rectangleUpdated(const QRect &rectangle);
 
     Q_SIGNAL void errorMessage(unsigned int error);
 
+    RdpView *rdpView()
+    {
+        return m_view;
+    };
+
 private:
     friend BOOL preConnect(freerdp *);
     friend BOOL postConnect(freerdp *);
@@ -120,6 +133,7 @@ private:
                                           DWORD);
     friend BOOL endPaint(rdpContext *);
     friend BOOL resizeDisplay(rdpContext *);
+    friend BOOL playSound(rdpContext *, const PLAY_SOUND_UPDATE *);
 
     void setState(State newState);
 
@@ -133,6 +147,7 @@ private:
 
     bool onEndPaint();
     bool onResizeDisplay();
+    bool onPlaySound();
 
     void run();
 
@@ -142,6 +157,7 @@ private:
 
     freerdp *m_freerdp = nullptr;
     RdpContext *m_context = nullptr;
+    std::unique_ptr<RdpClipboard> m_clipboard;
 
     State m_state = State::Initial;
 
